@@ -16,6 +16,14 @@ static char copyright[] =
 "This product includes software developed by the University of Michigan, Merit"
 "Network, Inc., and their contributors.";
 
+/* inet_ntop() and inet_pton() require at least Windows Vista. */
+#ifdef _WIN32
+# undef WINVER
+# undef _WIN32_WINNT
+# define WINVER 0x0600
+# define _WIN32_WINNT 0x0600
+#endif
+
 #include <assert.h> /* assert */
 #include <ctype.h> /* isdigit */
 #include <errno.h> /* errno */
@@ -26,7 +34,6 @@ static char copyright[] =
 #include <string.h> /* memcpy, strchr, strlen */
 #include <sys/types.h> /* BSD: for inet_addr */
 #include <sys/socket.h> /* BSD, Linux: for inet_addr */
-#include <netinet/in.h> /* BSD, Linux: for inet_addr */
 #include <arpa/inet.h> /* BSD, Linux, Solaris: for inet_addr */
 
 #include "patricia.h"
@@ -61,50 +68,13 @@ comp_with_mask (void *addr, void *dest, u_int mask)
     return (0);
 }
 
-/* inet_pton substitute implementation
- * Uses inet_addr to convert an IP address in dotted decimal notation into 
- * unsigned long and copies the result to dst.
- * Only supports AF_INET.  Follows standard error return conventions of 
- * inet_pton.
- */
-int
-inet_pton (int af, const char *src, void *dst)
-{
-    u_long result;  
-
-    if (af == AF_INET) {
-	result = inet_addr(src);
-	if (result == -1)
-	    return 0;
-	else {
-			memcpy (dst, &result, 4);
-	    return 1;
-		}
-	}
-#ifdef NT
-#ifdef HAVE_IPV6
-	else if (af == AF_INET6) {
-		struct in6_addr Address;
-		return (inet6_addr(src, &Address));
-	}
-#endif /* HAVE_IPV6 */
-#endif /* NT */
-#ifndef NT
-    else {
-
-	errno = EAFNOSUPPORT;
-	return -1;
-    }
-#endif /* NT */
-}
-
 /* this allows imcomplete prefix */
 int
 my_inet_pton (int af, const char *src, void *dst)
 {
     if (af == AF_INET) {
         int i, c, val;
-        u_char xp[4] = {0, 0, 0, 0};
+        u_char xp[sizeof(struct in_addr)] = {0, 0, 0, 0};
 
         for (i = 0; ; i++) {
 	    c = *src++;
@@ -125,7 +95,7 @@ my_inet_pton (int af, const char *src, void *dst)
 	    if (i >= 3)
 		return (0);
         }
-	memcpy (dst, xp, 4);
+	memcpy (dst, xp, sizeof(struct in_addr));
         return (1);
 #ifdef HAVE_IPV6
     } else if (af == AF_INET6) {
@@ -139,6 +109,8 @@ my_inet_pton (int af, const char *src, void *dst)
     }
 }
 
+#define PATRICIA_MAX_THREADS		16
+
 /* 
  * convert prefix information to ascii string with length
  * thread safe and (almost) re-entrant implementation
@@ -152,7 +124,7 @@ prefix_toa2x (prefix_t *prefix, char *buff, int with_len)
     if (buff == NULL) {
 
         struct buffer {
-            char buffs[16][48+5];
+            char buffs[PATRICIA_MAX_THREADS][48+5];
             u_int i;
         } *buffp;
 
@@ -169,11 +141,11 @@ prefix_toa2x (prefix_t *prefix, char *buff, int with_len)
 	    return (NULL);
 	}
 
-	buff = buffp->buffs[buffp->i++%16];
+	buff = buffp->buffs[buffp->i++%PATRICIA_MAX_THREADS];
     }
     if (prefix->family == AF_INET) {
 	u_char *a;
-	assert (prefix->bitlen <= 32);
+	assert (prefix->bitlen <= sizeof(struct in_addr) * 8);
 	a = prefix_touchar (prefix);
 	if (with_len) {
 	    sprintf (buff, "%d.%d.%d.%d/%d", a[0], a[1], a[2], a[3],
@@ -189,7 +161,7 @@ prefix_toa2x (prefix_t *prefix, char *buff, int with_len)
 	char *r;
 	r = (char *) inet_ntop (AF_INET6, &prefix->add.sin6, buff, 48 /* a guess value */ );
 	if (r && with_len) {
-	    assert (prefix->bitlen <= 128);
+	    assert (prefix->bitlen <= sizeof(struct in6_addr) * 8);
 	    sprintf (buff + strlen (buff), "/%d", prefix->bitlen);
 	}
 	return (buff);
@@ -220,32 +192,32 @@ prefix_t *
 New_Prefix2 (int family, void *dest, int bitlen, prefix_t *prefix)
 {
     int dynamic_allocated = 0;
-    int default_bitlen = 32;
+    int default_bitlen = sizeof(struct in_addr) * 8;
 
 #ifdef HAVE_IPV6
     if (family == AF_INET6) {
-        default_bitlen = 128;
+        default_bitlen = sizeof(struct in6_addr) * 8;
 	if (prefix == NULL) {
-            prefix = calloc(1, sizeof (prefix6_t));
+            prefix = calloc(1, sizeof (prefix_t));
 	    dynamic_allocated++;
 	}
-	memcpy (&prefix->add.sin6, dest, 16);
+	memcpy (&prefix->add.sin6, dest, sizeof(struct in6_addr));
     }
     else
 #endif /* HAVE_IPV6 */
     if (family == AF_INET) {
-		if (prefix == NULL) {
+	if (prefix == NULL) {
 #ifndef NT
             prefix = calloc(1, sizeof (prefix4_t));
 #else
-			//for some reason, compiler is getting
-			//prefix4_t size incorrect on NT
-			prefix = calloc(1, sizeof (prefix_t)); 
+	    //for some reason, compiler is getting
+	    //prefix4_t size incorrect on NT
+	    prefix = calloc(1, sizeof (prefix_t)); 
 #endif /* NT */
 		
-			dynamic_allocated++;
-		}
-		memcpy (&prefix->add.sin, dest, 4);
+	    dynamic_allocated++;
+	}
+	memcpy (&prefix->add.sin, dest, sizeof(struct in_addr));
     }
     else {
         return (NULL);
@@ -282,7 +254,7 @@ ascii2prefix (int family, char *string)
     char save[MAXLINE];
 
     if (string == NULL)
-		return (NULL);
+	return (NULL);
 
     /* easy way to handle both families */
     if (family == 0) {
@@ -293,50 +265,50 @@ ascii2prefix (int family, char *string)
     }
 
     if (family == AF_INET) {
-		maxbitlen = 32;
+	maxbitlen = sizeof(struct in_addr) * 8;
     }
 #ifdef HAVE_IPV6
     else if (family == AF_INET6) {
-		maxbitlen = 128;
+	maxbitlen = sizeof(struct in6_addr) * 8;
     }
 #endif /* HAVE_IPV6 */
 
     if ((cp = strchr (string, '/')) != NULL) {
-		bitlen = atol (cp + 1);
-		/* *cp = '\0'; */
-		/* copy the string to save. Avoid destroying the string */
-		assert (cp - string < MAXLINE);
-		memcpy (save, string, cp - string);
-		save[cp - string] = '\0';
-		string = save;
-		if (bitlen < 0 || bitlen > maxbitlen)
-			bitlen = maxbitlen;
-		}
-		else {
-			bitlen = maxbitlen;
-		}
+	bitlen = atol (cp + 1);
+	/* *cp = '\0'; */
+	/* copy the string to save. Avoid destroying the string */
+	assert (cp - string < MAXLINE);
+	memcpy (save, string, cp - string);
+	save[cp - string] = '\0';
+	string = save;
+	if (bitlen < 0 || bitlen > maxbitlen)
+	    bitlen = maxbitlen;
+	}
+	else {
+	    bitlen = maxbitlen;
+	}
 
-		if (family == AF_INET) {
-			if ((result = my_inet_pton (AF_INET, string, &sin)) <= 0)
-				return (NULL);
-			return (New_Prefix (AF_INET, &sin, bitlen));
-		}
+	if (family == AF_INET) {
+	    if ((result = my_inet_pton (AF_INET, string, &sin)) <= 0)
+		return (NULL);
+	    return (New_Prefix (AF_INET, &sin, bitlen));
+	}
 
 #ifdef HAVE_IPV6
-		else if (family == AF_INET6) {
+	else if (family == AF_INET6) {
 // Get rid of this with next IPv6 upgrade
 #if defined(NT) && !defined(HAVE_INET_NTOP)
-			inet6_addr(string, &sin6);
-			return (New_Prefix (AF_INET6, &sin6, bitlen));
+	    inet6_addr(string, &sin6);
+	    return (New_Prefix (AF_INET6, &sin6, bitlen));
 #else
-			if ((result = inet_pton (AF_INET6, string, &sin6)) <= 0)
-				return (NULL);
+	    if ((result = inet_pton (AF_INET6, string, &sin6)) <= 0)
+		return (NULL);
 #endif /* NT */
-			return (New_Prefix (AF_INET6, &sin6, bitlen));
-		}
+	    return (New_Prefix (AF_INET6, &sin6, bitlen));
+	}
 #endif /* HAVE_IPV6 */
-		else
-			return (NULL);
+	else
+	    return (NULL);
 }
 
 prefix_t *
@@ -413,7 +385,7 @@ Clear_Patricia (patricia_tree_t *patricia, void_fn_t func)
     	    if (Xrn->prefix) {
 		Deref_Prefix (Xrn->prefix);
 		if (Xrn->data && func)
-	    	    func (Xrn->data);
+	    	    ((void (*)(void *)) func) (Xrn->data);
     	    }
     	    else {
 		assert (Xrn->data == NULL);
@@ -431,7 +403,7 @@ Clear_Patricia (patricia_tree_t *patricia, void_fn_t func)
             } else if (Xsp != Xstack) {
                 Xrn = *(--Xsp);
             } else {
-                Xrn = (patricia_node_t *) 0;
+                Xrn = NULL;
             }
         }
     }
@@ -460,7 +432,7 @@ patricia_process (patricia_tree_t *patricia, void_fn_t func)
     assert (func);
 
     PATRICIA_WALK (patricia->head, node) {
-	func (node->prefix, node->data);
+	((void (*)(prefix_t *, void *)) func) (node->prefix, node->data);
     } PATRICIA_WALK_END;
 }
 
@@ -475,7 +447,7 @@ patricia_walk_inorder(patricia_node_t *node, void_fn_t func)
     }
 
     if (node->prefix) {
-	func(node->prefix, node->data);
+	((void (*)(prefix_t *, void *)) func) (node->prefix, node->data);
 	n++;
     }
 	
@@ -513,7 +485,7 @@ patricia_search_exact (patricia_tree_t *patricia, prefix_t *prefix)
     	        fprintf (stderr, "patricia_search_exact: take right %s/%d\n", 
 	                 prefix_toa (node->prefix), node->prefix->bitlen);
 	    else
-    	        fprintf (stderr, "patricia_search_exact: take right at %d\n", 
+    	        fprintf (stderr, "patricia_search_exact: take right at %u\n", 
 			 node->bit);
 #endif /* PATRICIA_DEBUG */
 	    node = node->r;
@@ -524,7 +496,7 @@ patricia_search_exact (patricia_tree_t *patricia, prefix_t *prefix)
     	        fprintf (stderr, "patricia_search_exact: take left %s/%d\n", 
 	                 prefix_toa (node->prefix), node->prefix->bitlen);
 	    else
-    	        fprintf (stderr, "patricia_search_exact: take left at %d\n", 
+    	        fprintf (stderr, "patricia_search_exact: take left at %u\n", 
 			 node->bit);
 #endif /* PATRICIA_DEBUG */
 	    node = node->l;
@@ -539,7 +511,7 @@ patricia_search_exact (patricia_tree_t *patricia, prefix_t *prefix)
         fprintf (stderr, "patricia_search_exact: stop at %s/%d\n", 
 	         prefix_toa (node->prefix), node->prefix->bitlen);
     else
-        fprintf (stderr, "patricia_search_exact: stop at %d\n", node->bit);
+        fprintf (stderr, "patricia_search_exact: stop at %u\n", node->bit);
 #endif /* PATRICIA_DEBUG */
     if (node->bit > bitlen || node->prefix == NULL)
 	return (NULL);
@@ -594,7 +566,7 @@ patricia_search_best2 (patricia_tree_t *patricia, prefix_t *prefix, int inclusiv
     	        fprintf (stderr, "patricia_search_best: take right %s/%d\n", 
 	                 prefix_toa (node->prefix), node->prefix->bitlen);
 	    else
-    	        fprintf (stderr, "patricia_search_best: take right at %d\n", 
+    	        fprintf (stderr, "patricia_search_best: take right at %u\n", 
 			 node->bit);
 #endif /* PATRICIA_DEBUG */
 	    node = node->r;
@@ -605,7 +577,7 @@ patricia_search_best2 (patricia_tree_t *patricia, prefix_t *prefix, int inclusiv
     	        fprintf (stderr, "patricia_search_best: take left %s/%d\n", 
 	                 prefix_toa (node->prefix), node->prefix->bitlen);
 	    else
-    	        fprintf (stderr, "patricia_search_best: take left at %d\n", 
+    	        fprintf (stderr, "patricia_search_best: take left at %u\n", 
 			 node->bit);
 #endif /* PATRICIA_DEBUG */
 	    node = node->l;
@@ -625,7 +597,7 @@ patricia_search_best2 (patricia_tree_t *patricia, prefix_t *prefix, int inclusiv
         fprintf (stderr, "patricia_search_best: stop at %s/%d\n", 
 	         prefix_toa (node->prefix), node->prefix->bitlen);
     else
-        fprintf (stderr, "patricia_search_best: stop at %d\n", node->bit);
+        fprintf (stderr, "patricia_search_best: stop at %u\n", node->bit);
 #endif /* PATRICIA_DEBUG */
 
     if (cnt <= 0)
@@ -701,7 +673,7 @@ patricia_lookup (patricia_tree_t *patricia, prefix_t *prefix)
     	        fprintf (stderr, "patricia_lookup: take right %s/%d\n", 
 	                 prefix_toa (node->prefix), node->prefix->bitlen);
 	    else
-    	        fprintf (stderr, "patricia_lookup: take right at %d\n", node->bit);
+    	        fprintf (stderr, "patricia_lookup: take right at %u\n", node->bit);
 #endif /* PATRICIA_DEBUG */
 	    node = node->r;
 	}
@@ -713,7 +685,7 @@ patricia_lookup (patricia_tree_t *patricia, prefix_t *prefix)
     	        fprintf (stderr, "patricia_lookup: take left %s/%d\n", 
 	             prefix_toa (node->prefix), node->prefix->bitlen);
 	    else
-    	        fprintf (stderr, "patricia_lookup: take left at %d\n", node->bit);
+    	        fprintf (stderr, "patricia_lookup: take left at %u\n", node->bit);
 #endif /* PATRICIA_DEBUG */
 	    node = node->l;
 	}
@@ -761,7 +733,7 @@ patricia_lookup (patricia_tree_t *patricia, prefix_t *prefix)
             fprintf (stderr, "patricia_lookup: up to %s/%d\n", 
 	             prefix_toa (node->prefix), node->prefix->bitlen);
 	else
-            fprintf (stderr, "patricia_lookup: up to %d\n", node->bit);
+            fprintf (stderr, "patricia_lookup: up to %u\n", node->bit);
 #endif /* PATRICIA_DEBUG */
     }
 
@@ -1015,7 +987,7 @@ lookup_then_remove (patricia_tree_t *tree, char *string)
 {
     patricia_node_t *node;
 
-    if (node = try_search_exact (tree, string))
+    if ((node = try_search_exact (tree, string)))
         patricia_remove (tree, node);
 }
 
@@ -1033,6 +1005,7 @@ try_search_best (patricia_tree_t *tree, char *string)
         printf ("try_search_best: %s/%d found\n", 
 	        prefix_toa (node->prefix), node->prefix->bitlen);
     Deref_Prefix (prefix);
+    return (node);
 }
 
 /* } */
